# Get content from the config.json
$config = Get-Content -Path .\config.json | ConvertFrom-Json

$xu_location = $config.xu_location
$xu_server = $config.xu_server
$xu_port = $config.xu_port
$sas_uri = $config.sas_uri
$end_of_upload_api = $config.end_of_upload_api

$extraction_start_date = $config.extraction_start_date
$extraction_end_date = $config.extraction_end_date
$language = $config.language
$exchange_rate_type = $config.exchange_rate_type

# Create a new log file for every run
$logFile = ".\extraction.log"

if (Test-Path -Path .\$logFile -PathType Leaf) {
    Remove-Item .\$logFile
}

# Function for appending text to the log file
function WriteLog
{
Param ([string]$logText)
$timestamp = (Get-Date).toString("dd/MM/yyyy HH:mm:ss")
Add-content $logFile -value "$timestamp $logText"
Write-Output $logText
}

# Function for checking the error code of extractions
function CheckExtractionError
{
Param ([string]$extraction)
if ($LASTEXITCODE -ne 0) {
    WriteLog("$extraction failed with return code: $LASTEXITCODE")
    Set-Variable -Name "extraction_failed" -Value 1 -Scope global
}
}

# Check whether the process-mining-data-uploader.exe can be found
if (-not(Test-Path -Path .\process-mining-data-uploader.exe -PathType Leaf)) {
    WriteLog("No process-mining-data-uploader.exe found.")
    Exit
}

# If at least one extraction fails, do not upload to blob storage.
Set-Variable -Name "extraction_failed" -Value 0 -Scope global

# Extraction commands
## Transactional tables
# Sales document
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_VBAK" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('O2C_VBAK')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_VBAP" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('O2C_VBAP')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_VBEP"
CheckExtractionError('O2C_VBEP')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_VBKD"
CheckExtractionError('O2C_VBKD')

# Delivery
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_LIKP" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('O2C_LIKP')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_LIPS" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('O2C_LIPS')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_VBRK" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('O2C_VBRK')

# Billing
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_VBRP" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('O2C_VBRP')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_BKPF" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('O2C_BKPF')

# Accounting
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_BSAD" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('O2C_BSAD')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_BSID" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('O2C_BSID')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_BSE_CLR"
CheckExtractionError('O2C_BSE_CLR')

# Change log
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_CDHDR" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('O2C_CDHDR')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_CDPOS" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('O2C_CDPOS')

## Master data
# User
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_ADCP" 
CheckExtractionError('O2C_ADCP')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_ADRP" 
CheckExtractionError('O2C_ADRP')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_USR02" 
CheckExtractionError('O2C_USR02')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_USR21" 
CheckExtractionError('O2C_USR21')

# Customer
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_KNA1" 
CheckExtractionError('O2C_KNA1')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_T005T" -o "language=$language"
CheckExtractionError('O2C_T005T')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_T005U" -o "language=$language"
CheckExtractionError('O2C_T005U')

# Material
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_MAKT" -o "language=$language"
CheckExtractionError('O2C_MAKT')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_T001L"
CheckExtractionError('O2C_T001L')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_T006A" -o "language=$language"
CheckExtractionError('O2C_T006A')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_T023T" -o "language=$language"
CheckExtractionError('O2C_T023T')

# Organizational structure
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_T001"
CheckExtractionError('O2C_T001')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_T001W"
CheckExtractionError('O2C_T001W')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_TSPAT" -o "language=$language"
CheckExtractionError('O2C_TSPAT')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_TVGRT"
CheckExtractionError('O2C_TVGRT')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_TVKBT" -o "language=$language"
CheckExtractionError('O2C_TVKBT')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_TVKOT" -o "language=$language"
CheckExtractionError('O2C_TVKOT')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_TVST"
CheckExtractionError('O2C_TVST')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_TVTWT" -o "language=$language"
CheckExtractionError('O2C_TVTWT')

# Sales document
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_TVAKT" -o "language=$language"
CheckExtractionError('O2C_TVAKT')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_TVAPT" -o "language=$language"
CheckExtractionError('O2C_TVAPT')

# Block/cancellation reason
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_T008T" -o "language=$language"
CheckExtractionError('O2C_T008T')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_T041CT" -o "language=$language"
CheckExtractionError('O2C_T041CT')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_TVAGT" -o "language=$language"
CheckExtractionError('O2C_TVAGT')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_TVFST" -o "language=$language"
CheckExtractionError('O2C_TVFST')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_TVLST" -o "language=$language"
CheckExtractionError('O2C_TVLST')

# Currency
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_TCURF" -o "exchange_rate_type=$exchange_rate_type"
CheckExtractionError('O2C_TCURF')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_TCURR" -o "exchange_rate_type=$exchange_rate_type"
CheckExtractionError('O2C_TCURR')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "O2C_TCURX"
CheckExtractionError('O2C_TCURX')

# Upload to the blob storage only if extraction is successful.
if ($extraction_failed) {
    Exit
} else {
    .\process-mining-data-uploader.exe "$xu_location\o2c_output" $sas_uri $end_of_upload_api | tee $logFile
}
